import inspect
import logging
import sys
from abc import ABC, abstractmethod
from importlib import import_module

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Button, Submit
from django import forms
from django.conf import settings

logger = logging.getLogger(__name__)

LIST_STRUCTURE = 0
TREE_STRUCTURE = 1


class ToolForm(forms.Form):
    project_id = int

    def __init__(self, *args, **kwargs):
        super(ToolForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_group_wrapper_class = "row"
        self.helper.label_class = "col-sm-offset-1 col-sm-2"
        self.helper.field_class = "col-md-8"
        self.helper.add_input(
            Button(
                "back",
                "Back",
                css_class="btn-secondary",
                onClick="javascript:history.go(-1);",
            )
        )
        self.helper.add_input(Submit("submit", "Add Tool"))

    def clean(self):
        from openlink.core.models import Tool

        cleaned_data = super().clean()
        if "name" in cleaned_data:
            if Tool.objects.filter(
                name=cleaned_data["name"], project_id=self.project_id
            ).first():
                self.add_error("name", "a tool with the same name already exists ")
        return cleaned_data


class BasicObject:
    """Constructor method"""

    def __init__(self, id, name, inner_type, description=None):
        self.id = id
        self.name = name
        self.inner_type = inner_type
        self.description = description


class DataObject(BasicObject):
    """Constructor method"""

    def __init__(self, id, name, inner_type, description=None):
        super().__init__(id, name, inner_type, description)


class ContainerObject(BasicObject):
    """Constructor method"""

    def __init__(self, id, name, inner_type, description=None):
        super().__init__(id, name, inner_type, description)


class ContainerDataObject(DataObject, ContainerObject):
    """Constructor method"""

    def __init__(self, id, name, inner_type, description=None):
        super().__init__(id, name, inner_type, description)


class ToolConnector(ABC):
    """Parent class of all connectors, declares the basic functions of a connector."""

    @classmethod
    @abstractmethod
    def get_name(cls):
        """Return the connector name.

        Returns:
            str: Connector name.
        """
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def get_creation_form(cls):
        """Returns class used for connector creation form.

        Returns:
            class: Form class needed for connector creation form.
        """
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def get_edition_form(cls):
        """Returns class used for connector edition form.

        Returns:
            class: Form class needed for connector edit form.
        """
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def has_access_url(cls):
        """Return True if the mapping object can be accessed online, through an URL.

        Returns:
            bool: True if the mapping object be can be accessed through an URL.
        """
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def get_logo(cls):
        """Return the static path of the connector logo.

        Returns:
            str: Connector logo static path.
        """
        raise NotImplementedError()

    @classmethod
    def has_mapping_options(cls):
        """Return True if the connector has mapping options.

        Returns:
            bool: True if connector has specific mapping options.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_url_link_to_an_object(self, obj_type, obj_id):
        """Get url link for a given object id and type.

        Arguments:
            obj_type (str): type of the object
            obj_id (int): Id of an object
        Returns:
            str:
            url for a given object
        """
        raise NotImplementedError()


class Mapper(ToolConnector):
    """Parent class for data management connector, declares the minimum
    functions to map a data to openlink.
    """

    @classmethod
    @abstractmethod
    def get_supported_types(cls):
        """A list of tuples with singular ressources name and their
        nomenclature equivalent in Openlink.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_space_info(self, objects_id):
        """Retrieve the size from a list of objects.

        Arguments:
            objects_id (list or None): A list of objects id.
        Returns:
            int: total size of all input objects in bytes.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_investigation(self, object_id):
        """
        Retrieve an object that can be mapped to an Openlink Investigation.

        Arguments:
            object_id (str): Id of an object.
        Returns:
            BasicObject: Instance of a BasicObject containing object id,
            name and description.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_study(self, object_id):
        """Retrieve an object that can be mapped to an Openlink Study.

        Arguments:
            object_id (str): Id of an object.
        Returns:
            BasicObject: name and description.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_assay(self, object_id):
        """Retrieve an object that can be mapped to an Openlink Assay.

        Arguments:
            object_id (str): Id of an object.
        Returns:
            BasicObject: Instance of a BasicObject containing object id,
            name and description.
        """
        pass

    @abstractmethod
    def get_dataset(self, object_id):
        """Retrieve an object that can be mapped to an Openlink Dataset.

        Arguments:
            object_id (str): Id of an object.
        Returns:
            BasicObject: Instance of a BasicObject containing object id,
            name and description.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_investigations(self):
        """Retrieve a list of objects that can be mapped to
            an Openlink Investigation.

        Returns:
            list:  List of Instance of a BasicObject containing object id,
            name and description.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_studies(self, reference_mapping, container):
        """Retrieve a list of objects that can be mapped to an Openlink Studies.

        Arguments:
            reference_mapping (BasicObject): Instance of a BasicObject, needed for the retrival of the objects sought.
            container (BasicObject): Instance of a BasicObject, container of the objects sought.
        Returns:
            list: List of Instance of a BasicObject containing object id,
            name and description.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_assays(self, reference_mapping, container):
        """Retrieve a list of objects that can be mapped to an Openlink Assays.

        Arguments:
            reference_mapping (BasicObject): Instance of a BasicObject, needed for the retrival of the objects sought.
            container (BasicObject): Instance of a BasicObject, container of the objects sought.
        Returns:
            list: List of Instance of a BasicObject containing object id,
            name and description.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_datasets(self, reference_mapping, container):
        """Retrieve a list of objects that can be mapped to an Openlink Datasets.

        Arguments:
            reference_mapping (BasicObject): Instance of a BasicObject, needed for the retrival of the objects sought.
            container (BasicObject): Instance of a BasicObject, container of the objects sought.
        Returns:
            list: List of Instance of a BasicObject containing object id,
            name and description.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_data_object(self, data_type, object_id):
        """Launch a function depending of a given data_type,
        to retrieve an object that can be mapped.

        Arguments:
            data_type (class): Openlink type of an object.
            object_id (str): Id of an object.
        Returns:
            list: List of Instance of a BasicObject containing object id,
            name and description.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_data_objects(self, data_type, reference_mapping=None, container=None):
        """Launch a function depending of a given data_type,
        to retrieve a list of objects that can be mapped.

        Arguments:
            data_type (class): Openlink type of an object.
            reference_mapping (BasicObject): Instance of a BasicObject, needed for the retrival of the objects sought.
            container (BasicObject): Instance of a BasicObject, container of the objects sought.
        Returns:
            list: List of Instance of a BasicObject containing object id,
            name and description.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_plural_form_of_type(self, data_type):
        """Get a plural form of a type of data from Openlink objects.

        Arguments:
            data_type (class): Openlink type of an object.
        Return:
            str: Plural form of a type of data.
        """
        raise NotImplementedError()

    @abstractmethod
    def get_information(self, type, id):
        """retrieves different information from the connector session (author, tags, name).

        Arguments:
            type (str): Openlink type of an object.
            id (str): Id of an object.
        Returns:
            dict: Info about the object, such as tags and user
        """
        raise NotImplementedError()

    @abstractmethod
    def download(self, object_id, path):
        """Download data from an object link in openlink path.

        Arguments:
           object_id (str): Id of an object.
           path (str): Path to where the downloaded files will be stored.
        """
        raise NotImplementedError()


class Publisher(ToolConnector):
    """Parent class for data publishing connectors, declares the minimum
    functions to publish a dataset.
    This class can be modified after each new publisher added.
    """

    @abstractmethod
    def create_empty_depo(self):
        """Initiate a new depository in the publisher.

        Returns:
            json: Json structure sent by api connector requests.
        """
        raise NotImplementedError()

    @abstractmethod
    def add_file_to_depo(self, json, path_to_file):
        """adding file in depository

        Arguments:
            json (json): Json structure sent by api connector requests.
            path_to_file (str): path to the file downloaded by Openlink.
        Returns:
            json: Json structure sent by api connector requests.
        """
        raise NotImplementedError()

    @abstractmethod
    def add_metadata_to_depo(self, meta, jsonr):
        """adding metadata to depository

        Arguments:
            meta (dict): Dictionary of metadata specific of an object from a connector.
            jsonr (str): link to the depo, retrieved from a json object.
        Returns:
            json: Json structure sent by api connector requests.
        """
        raise NotImplementedError()

    @abstractmethod
    def publish_depo(self, json):
        """publish the depository.

        Arguments:
            json (json): Json structure holding a usefull link to publish data.
        Returns:
            json: Json structure sent by api connector requests.
        """
        raise NotImplementedError()


class AuthentificationError(Exception):
    def __init__(self, tool, invalid_item):
        self.message = "invalid " + tool.get_name() + " " + invalid_item
        super().__init__(self.message)


class NotFoundError(Exception):
    def __init__(self, tool):
        self.message = "the " + tool.get_name + " object not found "
        super().__init__(self.message)


class ToolUnreachableError(Exception):
    def __init__(self, tool):
        self.message = (
            tool.get_name() + " server temporarily unavailable, try again later "
        )
        super().__init__(self.message)


class permissionError(Exception):
    def __init__(self, tool):
        self.message = "you do not have the necessary permission for this action"
        super().__init__(self.message)


class defaultError(Exception):
    def __init__(self, tool):
        self.message = "an error has occured on " + tool.get_name()
        super().__init__(self.message)


def find_connectors_in_module(module):
    connectors = []
    for name, obj in inspect.getmembers(module):
        if (
            inspect.isclass(obj)
            and issubclass(obj, ToolConnector)
            and obj is not ToolConnector
            and obj is not Publisher
            and obj is not Mapper
        ):
            connectors.append(obj)
        if inspect.ismodule(obj) and module.__name__ in obj.__name__:
            connectors.extend(find_connectors_in_module(obj))

    return connectors


def get_connectors():
    """Retrieves dynamically all implementation of connectors in apps"""
    connectors = []

    for app_module_name in settings.INSTALLED_APPS:
        connectors_module = None
        connectors_module_name = f"{app_module_name}.connectors"
        if connectors_module_name in sys.modules:
            connectors_module = sys.modules[connectors_module_name]
        else:
            try:
                connectors_module = import_module(connectors_module_name)
            except ModuleNotFoundError:
                pass
                # logger.debug(f'No connectors module in {app_module_name}')

        connectors.extend(find_connectors_in_module(connectors_module))

    return connectors


def get_connector_class(connector_name):
    for connector in get_connectors():
        if connector.__name__ == connector_name:
            return connector

    return None


def get_connector(tool):
    connector_class = get_connector_class(tool.connector)
    return connector_class(tool)



import logging

from django.contrib import messages
from openlink.core.connector import DataObject
from openlink.core.models import Mapping

logger = logging.getLogger(__name__)


def create_children_mappings(
    request, tool, parent_openlink_object, reference_mapping, container, author
):
    data_type = type(parent_openlink_object)
    data_type_child = data_type.get_child_type()
    connector = tool.get_connector()
    children_data_objects = connector.get_data_objects(
        data_type_child.__name__.lower(), reference_mapping, container
    )
    for child_data_object in children_data_objects:
        if not isinstance(child_data_object, DataObject):
            next
        openlink_object = data_type_child.create_instance(child_data_object, author)
        openlink_object.save()
        messages.info(
            request,
            str(data_type_child.__name__ + " ")
            + str(openlink_object.name)
            + " created ",
        )
        data_type.link_child_to_parent(parent_openlink_object, openlink_object)
        Mapping_openlink_object = Mapping.create_instance(
            openlink_object, child_data_object, tool, author
        )
        Mapping_openlink_object.save()
        messages.info(
            request, "Mapping " + str(Mapping_openlink_object.name) + " created "
        )
        child_type = data_type_child.get_child_type()
        if child_type is not None:
            create_children_mappings(
                request,
                tool,
                openlink_object,
                reference_mapping,
                child_data_object,
                author,
            )


def add_number_at_end_if_name_exist(object_name, update_object):
    if any(
        d.name == object_name
        for d in update_object.get_parent(update_object.id).first().items
    ):
        i = 1
        name = object_name + " (" + str(i) + ")"
        while any(
            d.name == name
            for d in update_object.get_parent(update_object.id).first().items
        ):
            i += 1
            name = object_name + " (" + str(i) + ")"
        new_name = name
    else:
        new_name = object_name
    return new_name



import json
import logging

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Button, Submit
from django.apps import apps
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseForbidden, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from django.utils import timezone
from django.views.decorators.http import require_POST
from django.views.generic.edit import UpdateView
from guardian.decorators import permission_required_or_403
from openlink.core.connector import Mapper, Publisher
from openlink.core.forms import (
    AssayForm,
    DatasetForm,
    InvestigationForm,
    ProjectForm,
    SelectAssayForm,
    SelectStudyForm,
    StudyForm,
)
from openlink.core.models import (
    Assay,
    Dataset,
    Investigation,
    Mapping,
    Profile,
    Project,
    Study,
    Tool,
)
from openlink.core.views import projects

logger = logging.getLogger(__name__)


def get_items_id(obj, list_save):
    # Get items structure
    if hasattr(obj, "items"):
        for item in obj.items:
            list_save.append(item)
            get_items_id(item, list_save)
        return list_save


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def add_investigation(request, project_id, **kwargs):
    # Display an empty form to add a new investigation

    project = get_object_or_404(Project, pk=project_id)
    if request.method == "GET":
        items = {}
        items["project"] = project
        return render(
            request,
            "new_item/new_item.html",
            {
                "form": InvestigationForm(),
                "project": project,
                "parent": project,
                "item": "investigation",
            },
        )
    elif request.method == "POST":
        form = InvestigationForm(request.POST)
        form.parent_id = project_id
        if not form.is_valid():
            # Display the same empty investigation form if form is not valid
            return render(
                request,
                "new_item/new_item.html",
                {
                    "form": form,
                    "project": project,
                    "parent": project,
                    "item": "investigation",
                },
            )
        profile = get_object_or_404(Profile, user=request.user)

        investigation = Investigation.create(
            form=form, profile=profile, parent_id=project_id
        )

        messages.info(request, "Investigation " + str(investigation.name) + " created")

        return HttpResponseRedirect(reverse("core:projects-details", args=[project_id]))


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def add_study(request, investigation_id, project_id):
    # Display an empty form to add a new study  in a given project ID
    investigation = get_object_or_404(Investigation, pk=investigation_id)
    project = get_object_or_404(Project, pk=project_id)
    if request.method == "GET":
        return render(
            request,
            "new_item/new_item.html",
            {
                "form": StudyForm(),
                "project": project,
                "investigation": investigation,
                "parent": investigation,
                "item": "study",
            },
        )
    elif request.method == "POST":
        form = StudyForm(request.POST)
        form.parent_id = investigation_id
        if not form.is_valid():
            # Display the same empty study form if form is not valid
            return render(
                request,
                "new_item/new_item.html",
                {
                    "form": form,
                    "project": project,
                    "investigation": investigation,
                    "parent": investigation,
                    "item": "study",
                },
            )

        profile = get_object_or_404(Profile, user=request.user)
        study = Study.create(form=form, profile=profile, parent_id=investigation_id)

        messages.info(request, "Study " + str(study.name) + " created")

        # Add the new study in the given investigation model instance

        return HttpResponseRedirect(reverse("core:projects-details", args=[project_id]))


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def select_study_to_create_assay(request, project_id, investigation_id):
    # Display a form in order to select a study, in which the user want to create a new assay
    if request.method == "POST":
        study_id = request.POST["studies_name"]
        project = get_object_or_404(Project, pk=project_id)
        investigation = get_object_or_404(Investigation, pk=investigation_id)
        study = get_object_or_404(Study, pk=study_id)
        return render(
            request,
            "new_item/new_item.html",
            {
                "form": AssayForm(),
                "study": study,
                "investigation": investigation,
                "project": project,
                "parent": study,
                "item": "assay",
            },
        )
    else:

        investigation = get_object_or_404(Investigation, pk=investigation_id)
        project = get_object_or_404(Project, pk=project_id)
        return render(
            request,
            "new_item/form_choose_study.html",
            {
                "form": SelectStudyForm(instance=investigation),
                "investigation": investigation,
                "project": project,
            },
        )


@login_required
@require_POST
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def add_assay(request, project_id, investigation_id, study_id):
    # Save the new assay with data from the assay form
    form = AssayForm(request.POST)
    study = get_object_or_404(Study, id=study_id)
    form.parent_id = study_id
    if not form.is_valid():
        project = get_object_or_404(Project, pk=project_id)
        investigation = get_object_or_404(Investigation, pk=investigation_id)
        return render(
            request,
            "new_item/new_item.html",
            {
                "form": form,
                "study": study,
                "investigation": investigation,
                "project": project,
                "parent": study,
                "item": "assay",
            },
        )

    profile = get_object_or_404(Profile, user=request.user)
    assay = Assay.create(form, profile, study.id)

    messages.info(request, "Assay " + str(assay.name) + " created")

    return HttpResponseRedirect(reverse("core:projects-details", args=[project_id]))


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def select_assay_to_create_dataset(request, project_id, investigation_id):
    # Display a form in order to select a assay, in which the user want to create a new dataset
    if request.method == "POST":
        assay_id = request.POST["assays_name"]
        project = get_object_or_404(Project, pk=project_id)
        investigation = get_object_or_404(Investigation, pk=investigation_id)
        assay = get_object_or_404(Assay, pk=assay_id)
        return render(
            request,
            "new_item/new_item.html",
            {
                "form": DatasetForm(),
                "assay": assay,
                "investigation": investigation,
                "project": project,
                "parent": assay,
                "item": "dataset",
            },
        )
    else:

        project = get_object_or_404(Project, pk=project_id)
        investigation = get_object_or_404(Investigation, pk=investigation_id)
        return render(
            request,
            "new_item/form_choose_assay.html",
            {
                "form": SelectAssayForm(instance=investigation),
                "investigation": investigation,
                "project": project,
            },
        )


@login_required
@require_POST
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def add_dataset(request, project_id, investigation_id, assay_id):
    # Save the new dataset with data from the dataset form
    form = DatasetForm(request.POST)
    assay = get_object_or_404(Assay, id=assay_id)
    form.parent_id = assay_id
    if not form.is_valid():
        project = get_object_or_404(Project, pk=project_id)
        investigation = get_object_or_404(Investigation, pk=investigation_id)
        return render(
            request,
            "new_item/new_item.html",
            {
                "form": form,
                "assay": assay,
                "investigation": investigation,
                "project": project,
                "parent": assay,
                "item": "dataset",
            },
        )

    profile = get_object_or_404(Profile, user=request.user)
    dataset = Dataset.create(form, profile, assay.id)

    messages.info(request, "Dataset " + str(dataset.name) + " created")

    return HttpResponseRedirect(reverse("core:projects-details", args=[project_id]))


@login_required
def get_json_object(request, *args, **kwargs):
    # Build json data to display from json connector
    tool_id = kwargs["tool_id"]
    object_id = kwargs["map_id"]
    data_type = kwargs["data_type"]
    json_data = []
    tool = get_object_or_404(Tool, id=tool_id)
    connector = tool.get_connector()
    elements_json = connector.get_dir(object_id)
    for element in elements_json:
        if data_type == "dataset":
            if element.__class__.__name__ == "DataObject":
                json_data.append(
                    {
                        "id": element.id,
                        "parent": object_id,
                        "text": element.name,
                        "icon": "jstree-file",
                    }
                )
            elif element.__class__.__name__ == "ContainerDataObject":
                json_data.append(
                    {
                        "id": element.id,
                        "parent": object_id,
                        "text": element.name,
                        "icon": "jstree-folder",
                    }
                )
        else:
            if element.__class__.__name__ == "DataObject":
                json_data.append(
                    {
                        "id": element.id,
                        "parent": object_id,
                        "text": element.name,
                        "icon": "jstree-file",
                        "state": {"checkbox_disabled": True},
                    }
                )
            elif element.__class__.__name__ == "ContainerDataObject":
                json_data.append(
                    {
                        "id": element.id,
                        "parent": object_id,
                        "text": element.name,
                    }
                )
    json_tree = json.dumps(json_data)

    return render(request, "tools/json_list.html", {"json_tree": json_tree})


def get_plural_form_of_type(type):
    plural_form = {
        "investigation": "investigations",
        "study": "studies",
        "assay": "assays",
        "dataset": "datasets",
    }
    return plural_form[type]


class ItemUpdate(UpdateView):
    # Generic class for item update
    template_name = "item_update/item_update_form.html"

    def __init__(self, *args, **kwargs):
        super(ItemUpdate, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_group_wrapper_class = "row"
        self.helper.label_class = "col-sm-offset-1 col-sm-2"
        self.helper.field_class = "col-md-8"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["name"] = self.object.name
        context["id"] = self.object.id
        context["data_type"] = str(self.object.type)
        obj = apps.get_model("core", str(self.object.type))
        investigation = obj.get_investigation(self.object.id).first()
        context["item"] = get_object_or_404(obj, id=self.object.id)
        if obj == Dataset:
            assay = get_object_or_404(Assay, datasets__id=self.object.id)
            context["assay_linked"] = Mapping.objects.filter(
                foreign_id_obj__assay__id=assay.id
            ).exists()
        if context["data_type"] != "project":
            project = obj.get_project(self.object.id).first()
            context["project"] = project
            context["investigation"] = investigation
            self.request.session["reference_data_type"] = self.object.type
            context["request"] = self.request
            context["progress_bar"], context["space_info_list"] = projects.progressbar(
                context["item"], project
            )
            items_mapping = Mapping.objects.filter(foreign_id_obj=context["item"])
            mappings = []
            for item_mapping in items_mapping:
                mapping = []
                connector = item_mapping.connector_class
                if issubclass(connector.__class__, Mapper):
                    supported_types = item_mapping.get_supported_types_plural
                else:
                    supported_types = []
                mapping.append(item_mapping)
                mapping.append(supported_types)
                mappings.append(mapping)
            context["mappings"] = mappings
            context["nb_mapper"] = 0
            context["nb_publisher"] = 0
            for value in items_mapping:
                if (
                    issubclass(
                        Tool.objects.get(id=value.tool_id.id).get_connector().__class__,
                        Publisher,
                    )
                    is True
                ):
                    context["nb_publisher"] += 1
                else:
                    context["nb_mapper"] += 1
            context["investigation_linked"] = Mapping.objects.filter(
                foreign_id_obj__investigation__id=context["investigation"].id
            ).exists()
            return context
        else:
            context["item"] = get_object_or_404(Project, id=self.object.id)
            return context

    def form_valid(self, form, *args, **kwargs):
        project_id = self.kwargs["project_id"]
        project = get_object_or_404(Project, id=project_id)
        if not self.request.user.has_perm("change_project", project):
            return HttpResponseForbidden()
        item = form.save(commit=False)
        item.date_modified = timezone.now()
        item.save()
        messages.info(
            self.request, item.type.capitalize() + " " + str(item.name) + " updated"
        )
        return HttpResponseRedirect(self.request.get_full_path())

    def get_form(self, form_class=None):
        form = super().get_form(form_class)
        form.helper = FormHelper()
        form.helper.form_group_wrapper_class = "row"
        form.helper.label_class = "col-sm-offset-1 col-sm-2"
        form.helper.field_class = "col-md-8"
        form.helper.add_input(
            Button(
                "back",
                "Back",
                css_class="btn-secondary",
                onClick="javascript:history.go(-1);",
            )
        )
        form.helper.add_input(
            Submit(
                "submit",
                "Update " + self.object.type.capitalize(),
                css_class="btn-primary",
            )
        )
        return form


class ProjectUpdate(ItemUpdate):
    # Generic editing view for a given Project
    model = Project
    form_class = ProjectForm
    template_name = "item_update/project_update_form.html"
    pk_url_kwarg = "project_id"


class InvestigationUpdate(ItemUpdate):
    # Generic editing view for a given Investigation
    model = Investigation
    form_class = InvestigationForm
    pk_url_kwarg = "investigation_id"


class StudyUpdate(ItemUpdate):
    # Generic editing view for a given study
    model = Study
    form_class = StudyForm
    pk_url_kwarg = "study_id"


class AssayUpdate(ItemUpdate):
    # Generic editing view for a given Experience
    model = Assay
    form_class = AssayForm
    pk_url_kwarg = "assay_id"


class DatasetUpdate(ItemUpdate):
    # Generic editing view for a given Dataset
    model = Dataset
    form_class = DatasetForm
    pk_url_kwarg = "dataset_id"



from django import template

register = template.Library()


@register.filter(name="split")
def split(value, key):
    """
    Returns the value turned into a list.
    """
    return value.split(key)



"""
Django settings for openlink project.

Generated by 'django-admin startproject' using Django 3.0.3.

For more information on this file, see
https://docs.djangoproject.com/en/3.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.0/ref/settings/
"""

import logging
import os

import environ
import ldap
from django.contrib.messages import constants as messages
from django.core.management.utils import get_random_secret_key
from django_auth_ldap.config import LDAPSearch

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

env = environ.Env(
    # set casting, default value
    DEBUG=(bool, False),
    AUTH_LDAP=(bool, False),
    AUTH_OIDC=(bool, False),
)

env_paths = [
    environ.Path("/etc/openlink/openlink.env"),
]

if env.str('OPENLINK_ENV_PATH', default='') != '':
    env_paths.insert(0, environ.Path(env.str('OPENLINK_ENV_PATH')))

# Read in any environment files
for e in env_paths:
    try:
        e.file("")
        env.read_env(e())
    except FileNotFoundError:
        pass


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!

SECRET_KEY = env("SECRET_KEY", default=get_random_secret_key())

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = env("DEBUG")

ALLOWED_HOSTS = ["localhost"]


# Application definition

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "openlink.core",
    "django.contrib.admin",
    "crispy_forms",
    "django_filters",
    "widget_tweaks",
    "django_extensions",
    "django_json_ld",
    "popup_forms",
    "rest_framework",
    "django_rq",
    "guardian",
    "ckeditor",
    "debug_toolbar",
]

default_connectors = [
    'openlink.connectors.openlink_labguru',
    'openlink.connectors.openlink_omero',
    'openlink.connectors.openlink_seafile',
    'openlink.connectors.openlink_ssh',
    'openlink.connectors.openlink_zenodo'
]

INSTALLED_CONNECTORS = env.list("INSTALLED_CONNECTORS", default=default_connectors)
INSTALLED_APPS.extend(INSTALLED_CONNECTORS)

RQ_SHOW_ADMIN_LINK = True
RQ_QUEUES = {
    "default": {
        "HOST": "localhost",
        "PORT": 6379,
        "DB": 0,
    },
}

CRISPY_TEMPLATE_PACK = "bootstrap4"

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "debug_toolbar.middleware.DebugToolbarMiddleware",
]

ROOT_URLCONF = "openlink.urls"

LOGIN_REDIRECT_URL = "/projects"


MESSAGE_TAGS = {
    messages.ERROR: "danger",
}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            os.path.join(BASE_DIR, "openlink/templates"),
            os.path.join(BASE_DIR, "openlink/core/lib/tools/templates"),  # type: ignore
            os.path.join(BASE_DIR, "openlink/core/templates"),  # type: ignore
        ],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
            "libraries": {
                "split_tags": "openlink.core.templatetags.split",
                "param_tags": "openlink.core.templatetags.param",
                "url_link_tags": "openlink.core.templatetags.url_link",
                "humansize": "openlink.core.templatetags.humansize",
                "contains_assay_tags": "openlink.core.templatetags.contains_assay",
                "get_version": "openlink.core.templatetags.get_version",
                "plurals_type": "openlink.core.templatetags.plurals_type",
                "has_access_url": "openlink.core.templatetags.has_access_url",
                "get_oidc": "openlink.core.templatetags.get_oidc",
            },
        },
    },
]

WSGI_APPLICATION = "openlink.wsgi.application"


# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

if env.str('DATABASE_URL', default=''):
    DATABASES = {
        'default': env.db(),
    }
else:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(BASE_DIR, "db.sqlite3"),
        },
    }

# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "Europe/Paris"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

STATIC_URL = "/static/"
STATIC_ROOT = ".static"

INTERNAL_IPS = ("127.0.0.1",)

GRAPH_MODELS = {
    "all_applications": True,
    "group_models": True,
}

AUTH_PROFILE_MODULE = "openlink.core.profile"

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "filters": {
        "require_debug_true": {
            "()": "django.utils.log.RequireDebugTrue",
        },
    },
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "filters": ["require_debug_true"],
        },
    },
    "root": {
        "handlers": ["console"],
        "level": "DEBUG",
    },
    "loggers": {
        "mylogger": {
            "handlers": ["console"],
            "level": os.getenv("DJANGO_LOG_LEVEL", "INFO"),
            "propagate": True,
        },
    },
}


REST_FRAMEWORK = {
    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    "DEFAULT_PERMISSION_CLASSES": [
        "rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly"
    ]
}

AUTHENTICATION_BACKENDS = [
    "mozilla_django_oidc.auth.OIDCAuthenticationBackend",
    "django_auth_ldap.backend.LDAPBackend",
    "django.contrib.auth.backends.ModelBackend",
    "guardian.backends.ObjectPermissionBackend",
]

AUTH_LDAP = env("AUTH_LDAP")
if AUTH_LDAP:
    AUTH_LDAP_SERVER_URI = env("AUTH_LDAP_SERVER_URI")
    AUTH_LDAP_BIND_DN = env("AUTH_LDAP_BIND_DN")
    AUTH_LDAP_BIND_PASSWORD = env("AUTH_LDAP_BIND_PASSWORD")
    AUTH_LDAP_USER_SEARCH = LDAPSearch(
        env("AUTH_LDAP_SEARCH"), ldap.SCOPE_SUBTREE, "(uid=%(user)s)"
    )
    AUTH_LDAP_USER_ATTR_MAP = env.json("AUTH_LDAP_USER_ATTR_MAP")

AUTH_OIDC = env("AUTH_OIDC")
if AUTH_OIDC:
    OIDC_RP_CLIENT_ID = env("OIDC_RP_CLIENT_ID")
    OIDC_RP_CLIENT_SECRET = env("OIDC_RP_CLIENT_SECRET")
    OIDC_OP_AUTHORIZATION_ENDPOINT = env("OIDC_OP_AUTHORIZATION_ENDPOINT")
    OIDC_OP_TOKEN_ENDPOINT = env("OIDC_OP_TOKEN_ENDPOINT")
    OIDC_OP_USER_ENDPOINT = env("OIDC_OP_USER_ENDPOINT")
    OIDC_RP_SIGN_ALGO = env("OIDC_RP_SIGN_ALGO")
    OIDC_OP_JWKS_ENDPOINT = env("OIDC_OP_JWKS_ENDPOINT")


# AUTHENTICATION_BACKENDS = [
#     "django_auth_ldap.backend.LDAPBackend",
#     "django.contrib.auth.backends.ModelBackend", ]

# AUTH_LDAP_SERVER_URI = ""
# AUTH_LDAP_BIND_DN = ""

# AUTH_LDAP_BIND_PASSWORD = ""

# AUTH_LDAP_USER_SEARCH = LDAPSearch(

#     "ou=Utilisateurs,dc=igbmc,dc=u-strasbg,dc=fr", ldap.SCOPE_SUBTREE, "(uid=%(user)s)"

# )

# AUTH_LDAP_USER_ATTR_MAP = {
#     "first_name": "givenName",
#     "last_name": "sn",
#     "email": "mail",
# }

logger = logging.getLogger("django_auth_ldap")
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.DEBUG)

CKEDITOR_CONFIGS = {
    "zenodo": {
        "toolbar": "zenodo",
        "toolbar_zenodo": [
            {
                "name": "clipboard",
                "items": [
                    "PasteText",
                    "PasteFromWord",
                ],
            },
            {
                "name": "basicstyles",
                "items": [
                    "Bold",
                    "Italic",
                    "Strike",
                    "Subscript",
                    "Superscript",
                    "-",
                ],
            },
            {
                "name": "links",
                "items": [
                    "Link",
                    "Unlink",
                ],
            },
            {
                "name": "paragraph",
                "items": [
                    "NumberedList",
                    "BulletedList",
                    "Outdent",
                    "Indent",
                    "Blockquote",
                    "CodeSnippet",
                ],
            },
            {
                "name": "undo",
                "groups": ["clipboard", "undo"],
                "items": ["Undo", "Redo", "RemoveFormat"],
            },
            {"name": "insert", "items": ["SpecialChar", "Mathjax"]},
            {"name": "document", "items": ["Source"]},
            {
                "name": "tools",
                "items": [
                    "Maximize",
                ],
            },
        ],
        "extraPlugins": "codesnippet,mathjax",
        "mathJaxLib": "//cdn.mathjax.org/mathjax/2.2-latest/MathJax.js?config=TeX-AMS_HTML",
        "height": "100%",
        "width": "100%",
    },
}



#!/usr/bin/env python

from setuptools import find_packages, setup

from openlink import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='ifb-openlink',
    version=__version__,
    description='A dashboard that establish links between the structure of a research project and multiple data sources ',
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords='FAIR, open-data, data-managment-tools',
    url='https://gitlab.com/igbmc/openlink',
    project_urls={
        'Bug Tracker': 'https://gitlab.com/igbmc/openlink/-/issues',
        'Source Code': 'https://gitlab.com/igbmc/openlink',
    },
    author='Laurent Bouri, Julien Seiler',
    license='GNU General Public License v3 (GPLv3)',
    python_requires='>=3.6',
    packages=find_packages(),
    install_requires=[
        'django==3.0.7',
        'django-crispy-forms==1.9.1',
        'django-filter==2.3.0',
        'django-widget-tweaks==1.4.8',
        'django_extensions==2.2.9',
        'django_json_ld==0.0.4',
        'requests==2.23.0',
        'django-popup-forms==1.0.3',
        'wheel==0.34.2',
        'omero-py==5.6.0',
        'djangorestframework==3.11.1',
        'django-rq==2.4.0',
        'django-guardian==2.3.0',
        'django-ckeditor==6.1.0',
        'django-debug-toolbar==3.2.1',
        'paramiko==2.7.2',
        'django-environ==0.8.1',
    ],
    entry_points={
        'console_scripts': [
            'openlink = openlink:manage',
        ],
    },
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python :: 3',
        'Framework :: Django :: 3.0',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Topic :: Scientific/Engineering',
        'Topic :: System :: Systems Administration',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
    ]
)



import django_filters

from .models import Assay, Dataset, Investigation, Project, Study


class ProjectFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(lookup_expr="icontains")

    class Meta:
        model = Project
        fields = ["name"]


class InvestigationFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(lookup_expr="icontains")

    class Meta:
        model = Investigation
        fields = ["name"]


class DatasetFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(lookup_expr="icontains")

    class Meta:
        model = Dataset
        fields = ["name"]


class AssayFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(lookup_expr="icontains")

    class Meta:
        model = Assay
        fields = ["name"]


class StudyFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(lookup_expr="icontains")

    class Meta:
        model = Study
        fields = ["name"]



from django.contrib import admin

# from django.contrib.admin.options import get_content_type_for_model
# from django.urls import reverse
# from django.utils.html import format_html
# from django.utils.translation import ugettext
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.contrib.auth.models import User

from .models import (
    Assay,
    Dataset,
    Image,
    Investigation,
    Mapping,
    MappingParam,
    MappingProjectUser,
    Profile,
    Project,
    Study,
    Team,
    Tool,
    Toolparam,
)


# Define an inline admin descriptor for Profile model
# which acts a bit like a singleton
class ProfileInline(admin.StackedInline):
    model = Profile
    can_delete = False
    verbose_name_plural = "profile"


# Define a new User admin
class UserAdmin(BaseUserAdmin):
    inlines = (ProfileInline,)


# Re-register UserAdmin
admin.site.unregister(User)
admin.site.register(User, UserAdmin)


class ViewOnSiteModelAdmin(admin.ModelAdmin):
    class Media:
        css = {
            "all": (
                "https://maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css",
            )
        }


class ToolAdmin(admin.ModelAdmin):
    list_display = ("id", "name", "project")


class MappingAdmin(admin.ModelAdmin):
    list_display = ["name", "type", "tag_tool", "author"]


class ProjectAdmin(admin.ModelAdmin):
    list_display = ("name", "id")


class InvestigationAdmin(admin.ModelAdmin):
    list_display = ["name", "id", "get_project"]


class StudyAdmin(admin.ModelAdmin):
    list_display = ["name", "id", "get_project", "get_investigation"]


class AssayAdmin(admin.ModelAdmin):
    list_display = ["name", "id", "get_project", "get_investigation", "get_study"]


class DatasetAdmin(admin.ModelAdmin):
    list_display = [
        "name",
        "id",
        "get_project",
        "get_investigation",
        "get_study",
        "get_assay",
    ]


class MappingProjctUserAdmin(admin.ModelAdmin):
    list_display = ["user", "project", "role"]


admin.site.register(Project, ProjectAdmin)
admin.site.register(Investigation, InvestigationAdmin)
admin.site.register(Study, StudyAdmin)
admin.site.register(Assay, AssayAdmin)
admin.site.register(Dataset, DatasetAdmin)
admin.site.register(Image)
admin.site.register(Team)
admin.site.register(Mapping, MappingAdmin)
admin.site.register(Tool, ToolAdmin)
admin.site.register(Toolparam)
admin.site.register(MappingProjectUser, MappingProjctUserAdmin)
admin.site.register(MappingParam)
admin.site.register(Profile)



"""openlink URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
import debug_toolbar

# from openlink.core.admin import admin_site
from django.contrib import admin
from django.urls import include, path

admin.site.site_header = "OPENLINK Database administration"
admin.site.site_title = "OPENLINK administration"
admin.site.index_title = "OPENLINK Site Administration"

urlpatterns = [
    path("__debug__/", include(debug_toolbar.urls)),
    path("", include("openlink.core.urls")),
    path("admin/", admin.site.urls),
    path("accounts/", include("django.contrib.auth.urls")),
    path("django-rq/", include("django_rq.urls")),
    path('oidc/', include('mozilla_django_oidc.urls')),
]



from django import template

register = template.Library()


@register.filter(name="plurals_type")
def plurals_type(type):
    plural_form = {
        "investigation": "investigations",
        "study": "studies",
        "assay": "assays",
        "dataset": "datasets",
        "folder": "folders",
        "experiment": "experiments",
    }

    return plural_form[type]
